#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Thu Apr 27 14:42:46 2023

@author: mac
"""

#Asness Fight the FED-Model

#import libraries
import pandas as pd
import matplotlib.pyplot as plt
import os
from datetime import datetime as dt
import numpy as np
import statsmodels.api as sm
os.chdir('/Users/mac/Downloads/')

#%%
###FED-Model
#Import Data
SP_PE = pd.read_html('https://www.multpl.com/s-p-500-pe-ratio/table/by-month')[0].iloc[2:]
SP_P = pd.read_html('https://www.multpl.com/s-p-500-historical-prices/table/by-month')[0].iloc[2:]
Treasury = pd.read_csv('United States 10-Year Bond Yield Historical Data.csv')
CPI = pd.read_excel('SeriesReport-20230418090625_4530fc.xlsx').iloc[10:, 1:]

#Clean Data
#CPI
CPI.columns = CPI.iloc[0]
CPI = CPI.iloc[1:]
CPI = CPI[~CPI['Period'].str.contains('S01|S02')].iloc[:,2]
start_date = pd.to_datetime('1913-01-01')
end_date = pd.to_datetime('2023-03-01')
date_range = pd.date_range(start=start_date, end=end_date, freq='MS')
CPI.index = date_range
CPI = CPI.iloc[::-1].astype(float)
#S&P 
SP_PE['Value Value'] = SP_PE['Value Value'].str.replace(' estimate', '').astype(float)
SP_PE = SP_PE.iloc[:len(CPI)]
SP_PE.index = date_range[::-1]
SP_P = SP_P.iloc[:len(CPI)]
SP_P.index = date_range[::-1]
EPS = SP_P['Price Value'] / SP_PE['Value Value']

df_shiller = pd.read_excel('ie_data.xls', sheet_name= 'Data').iloc[6:]
df_shiller.columns = df_shiller.iloc[0]

df_shiller = df_shiller.iloc[1:-1]
df_shiller['Date'] = df_shiller['Date'].astype(str).str.replace('.', '-')

#%%
df_shiller['Date'] = pd.to_datetime(df_shiller['Date'], format = '%Y-%m')
df_shiller.index = df_shiller['Date']

df_shiller.iloc[:,1:] = df_shiller.iloc[:,1:].astype(float)

# Each EPS is then divided by the level of inflation



#%%
trailing_eps = pd.DataFrame(index = df_shiller.index, columns = ['e', 'g'])
trailing_cpi = pd.DataFrame(index = df_shiller.index, columns = ['cpi', 'g'])
trailing_p = pd.DataFrame(index = df_shiller.index, columns = ['p'])
trailing_y = pd.DataFrame(index = df_shiller.index, columns = ['y'])
for i in range(0,len(df_shiller)):
    trailing_eps['e'].iloc[i] = np.average(df_shiller['E'].iloc[i-120:i])
    trailing_cpi['cpi'].iloc[i] = np.average(df_shiller['CPI'].iloc[i-120:i])
    trailing_eps['g'].iloc[i] = (trailing_eps['e'].iloc[i]/trailing_eps['e'].iloc[i-120])**(1/10)-1
    trailing_cpi['g'].iloc[i] = (trailing_cpi['cpi'].iloc[i]/trailing_cpi['cpi'].iloc[i-120])**(1/10)-1
    trailing_p['p'].iloc[i] = np.average(df_shiller['P'].iloc[i-120:i])
    trailing_y['y'].iloc[i] = np.average(df_shiller['Rate GS10'].iloc[i-120:i])
#%% First regression
start = '1925-12'
end = '2022'

eps = trailing_eps['g'][(trailing_eps.index > start) & (trailing_eps.index < end)]
cpi = trailing_cpi['g'][(trailing_cpi.index > start) & (trailing_cpi.index < end)]

eps = pd.to_numeric(eps)
cpi = pd.to_numeric(cpi)
    
cpi = sm.add_constant(cpi)
y = np.asarray(eps)
x = np.asarray(cpi)
model = sm.OLS(y,x).fit()
print(model.summary())

#%%
start = '1880-12'
end = '2002'
#ep = trailing_eps['e']/df_shiller['P']
ep = 1/df_shiller['CAPE']
R10y = (df_shiller['Real Return'].iloc[:,0]) #10 year return

Y = df_shiller['Rate GS10']/100
gs10 = Y[(Y.index > start) & (Y.index < end)]
EP = ep[(ep.index > start) & (ep.index < end)]

#%%
y = R10y[(R10y.index > start) & (R10y.index < end)]
y = pd.to_numeric(y)
y = np.asarray(y)
EP = pd.to_numeric(EP)
x = np.asarray(sm.add_constant(EP))

model = sm.OLS(y,x).fit()
print(model.summary())

EP_Y = EP - gs10
EP_Y = EP_Y[(EP_Y.index > start) & (EP_Y.index < end)]
x = np.asarray(sm.add_constant(pd.to_numeric(EP_Y)))
model = sm.OLS(y,x).fit()
print(model.summary())

y = R10y[(R10y.index > start) & (R10y.index < end)]
y = pd.to_numeric(y)
y = np.asarray(y)
X = pd.concat([EP, gs10], axis=1)
x = np.asarray(sm.add_constant(X.values))

model = sm.OLS(y,x).fit()
print(model.summary())

#%%

df_shiller['Buyback sh'] = df_shiller['Buyback sh'].fillna(0)
df_shiller['Implied RP'] = (df_shiller['D']+df_shiller['Buyback sh'])/df_shiller['P']
df_shiller['Inflation'] = df_shiller['CPI']/df_shiller['CPI'].shift(12)-1
df_shiller['Real Rates'] = (1+df_shiller['Rate GS10']/100)/(1+df_shiller['Inflation'])-1

#The Risk Premium is typically measured bby solving the DDM for the discount rate. i.e. Dividend Yield
#DDM on the basis of consensus expectations objectively measures the rate of return that the marketplace
#appears to be demanding as compensation equity market risk
#As pointed out buy Damodaran, the buybacks are also important so they are added with the dividend to create the risk premium

#%%Regression
start = '2011-12'
end = '2022'
y = df_shiller['Implied RP'][(df_shiller['Implied RP'].index > start)&(df_shiller['Implied RP'].index < end)]
x = df_shiller[df_shiller.columns[df_shiller.columns.isin(['Real Rates','Inflation'])]]
x = x[(x.index > start) & (x.index < end)]
x = (sm.add_constant(x))
model = sm.OLS(y,x).fit()
print(model.summary())
#%%
duration = 1/(np.prod((1+y))**(1/len(y))-1)
print(duration)
i_dur = -duration*(1- 0.9114 +0.4270)
print(i_dur)
inf23 = df_shiller['Inflation'][df_shiller.index > '2020-12']
rr23 = df_shiller['Real Rates'][df_shiller.index > '2022-12']
#%%
implied_effect = i_dur * (inf23.iloc[-1] - inf23.iloc[0])
print(implied_effect)







