#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Fri May 12 17:33:18 2023

@author: mac
"""

import pandas as pd
from pandas.tseries.offsets import BMonthEnd
import numpy as np
import requests
import json
from datetime import datetime as dt
from datetime import timedelta
import os
import matplotlib.pyplot as plt
os.chdir('/Users/mac/Downloads/')

lst = pd.read_excel('REIT Residential.xlsx')
lst.drop(24, inplace = True)
lst.drop(19, inplace = True) #removing CSR as it is causing problems, we may have to circle back to it. Both  of the dropped stocks are CSR
i_s = []
c_f = []
mcap = []
d = []
price = []
for i in lst['Symbol']: #Creating a for-loop to gather income statements, cash-flow statements, market capitalization, dividends,  and prices
    inc = pd.DataFrame(requests.get('https://financialmodelingprep.com/api/v3/income-statement/'+i+'?period=quarter&limit=4000&apikey=e2d6d83834530d280c2409a4787cabd4').json())
    inc.index = pd.to_datetime(inc['date'])
    i_s.append(inc)
    cfl = pd.DataFrame(requests.get('https://financialmodelingprep.com/api/v3/cash-flow-statement/'+i+'?period=quarter&limit=4000&apikey=e2d6d83834530d280c2409a4787cabd4').json())
    cfl.index = pd.to_datetime(cfl['date'])
    c_f.append(cfl)
    cap = pd.DataFrame(requests.get('https://financialmodelingprep.com/api/v3/historical-market-capitalization/'+i+'?limit=25000&apikey=e2d6d83834530d280c2409a4787cabd4').json())
    cap.index = pd.to_datetime(cap['date'])
    mcap.append(cap)
    div = pd.DataFrame.from_dict(requests.get('https://financialmodelingprep.com/api/v3/historical-price-full/stock_dividend/'+i+'?apikey=e2d6d83834530d280c2409a4787cabd4').json())
    div = pd.json_normalize(div.iloc[:,1])
    div.index = pd.to_datetime(div['date'])
    d.append(div)
    p = pd.DataFrame.from_dict(requests.get('https://financialmodelingprep.com/api/v3/historical-price-full/'+i+'?serietype=line&apikey=e2d6d83834530d280c2409a4787cabd4').json())
    p = pd.json_normalize(p.iloc[:,1])
    p.index = pd.to_datetime(p['date'])
    price.append(p)
    print(i)


#%% Now focus on setting up the weights
#There should be a column for each of the market capitalizations, by using datetime index they should be placed according to it
#Manually find the longest time frame for the whole index
date_range = mcap[21].index.to_list()
column_names = lst['Symbol'].to_list()

market_caps = pd.DataFrame(columns = column_names, index = date_range)
for i in range(0,len(mcap)):
    mcap[i] = mcap[i].reindex(date_range)
    market_caps.iloc[:,i] = mcap[i]['marketCap']

#Now we have a possibility to create weighted averages, Now we simply need the earnings yield or P/E and the dividend yield

#%% Now focus on setting up the trailing eps for each of the stocks, this will give us a trailing P/E later

for t in range(0,len(i_s)):
    i_s[t]['t_eps'] = np.nan
    for i in range(0,len(i_s[t])-3):
        i_s[t]['t_eps'].iloc[i] = np.sum(i_s[t]['epsdiluted'].iloc[i:i+3])
    i_s[t] = i_s[t].iloc[:-3]
#%%
start_date = dt(2023,5,16).date()
end_date = dt(1983,1,1).date()
date_range = pd.date_range(start = end_date, end = start_date, freq = 'D')[::-1]


trailing_eps = pd.DataFrame(columns = column_names, index = date_range)
for i in range(len(i_s)):
    trailing_eps[column_names[i]] = i_s[i]['t_eps'].reindex(date_range).bfill() #filling empty cells
    
#%% Now we need the trailing dividends, this is found by combining the dividends that are all in one year from the date we are focusing on

#after calculating the trailing eps, we need to get a dataframe with both price and trailing eps, some trailing eps' has landed on weekends so we need to change those 
#the first mission would be to reindex both price and trailing eps

#we may need to use some sort of weighted index, as there is so few companies with a widely different timeline
#as one joins, it can pull down or pull up the earnings substantially



#%% #Creating trailing  dividends by summing all dividends for the enterity of a year for each observation date
all_d = pd.DataFrame(columns = column_names, index = date_range)
for n in range(0,len(d)):
    d[n]['trl div'] = np.nan
    for i, row in d[n].iterrows():       
        d[n].at[i, 'trl div'] = np.sum(d[n]['adjDividend'][(d[n].index <= row.name) & (d[n].index > row.name + timedelta(days=-365))])
    d[n] = d[n][d[n].index > d[n].iloc[-1].name + pd.DateOffset(years=1)]
trailing_d = pd.DataFrame(columns=column_names, index=date_range)
for i in range(len(d)):
    series = d[i]['trl div'].reindex(date_range).bfill()
    trailing_d.iloc[:, i] = series.values
#%% Creating End of Month Data For all to make it fit
date_index = price[21].index
monthly_date_range = pd.date_range(start=date_index[-1], end=date_index[0], freq='M') + BMonthEnd()
trailing_d = trailing_d.reindex(monthly_date_range)
trailing_eps = trailing_eps.reindex(monthly_date_range)

price_data = pd.DataFrame(columns = column_names, index = date_range)

for i in range(0,len(price)):
    series = price[i]['close'].reindex(date_range).bfill()
    price_data.iloc[:,i] = series.values

price_data = price_data.reindex(monthly_date_range)
market_caps = market_caps.reindex(date_range).bfill()
market_caps = market_caps.reindex(monthly_date_range)
#%% Creating an excel sheet
with pd.ExcelWriter('REIT index.xlsx') as writer:
    market_caps.to_excel(writer, sheet_name = 'Market Cap', index = True)
    price_data.to_excel(writer, sheet_name = 'Price', index = True)
    trailing_d.to_excel(writer, sheet_name = 'Trailing Dividends', index = True)
    trailing_eps.to_excel(writer, sheet_name = 'Trailing EPS')
#%%

